'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _InputLayer = require('../layer/InputLayer');

var _InputLayer2 = _interopRequireDefault(_InputLayer);

var _HiddenLayer = require('../layer/HiddenLayer');

var _HiddenLayer2 = _interopRequireDefault(_HiddenLayer);

var _OutputLayer = require('../layer/OutputLayer');

var _OutputLayer2 = _interopRequireDefault(_OutputLayer);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var DeepNetwork = function () {
    function DeepNetwork(layerConfig) {
        _classCallCheck(this, DeepNetwork);

        this.inputLayer = new _InputLayer2.default(layerConfig[0].size);

        if (layerConfig.length !== 3) {
            throw new Error('Having more or less than 1 hidden layer is not yet supported.');
        }

        var hiddenConfig = layerConfig[1];
        this.hiddenLayer = new _HiddenLayer2.default(hiddenConfig.size, this.inputLayer, hiddenConfig.activationFunction, hiddenConfig.learningRate);

        var outputConfig = layerConfig[2];
        this.outputLayer = new _OutputLayer2.default(outputConfig.size, this.hiddenLayer, outputConfig.activationFunction, outputConfig.learningRate);
        this.hiddenLayer.setOutputLayer(this.outputLayer);

        // console.log('hidden weights',this.hiddenLayer.weights);
        // console.log('output weights',this.outputLayer.weights);
    }

    _createClass(DeepNetwork, [{
        key: 'invoke',
        value: function invoke(inputs) {
            // for (var i = 0, len = inputs.length; i < len; i++) {
            //     if (!isFinite(inputs[i])) {
            //         throw new Error('Neural network input is not a finite number.');
            //     }
            // }

            this.inputLayer.feedForward(inputs);
            this.hiddenLayer.feedForward();
            var outputs = this.outputLayer.feedForward();

            for (var i = 0, len = outputs.length; i < len; i++) {
                if (!isFinite(outputs[i])) {
                    throw new Error('Neural network output is not a finite number.');
                }
            }

            return outputs;
        }
    }, {
        key: 'learn',
        value: function learn(targetOutputs) {
            this.outputLayer.backPropagateCalculateErrorGradient(targetOutputs);
            this.hiddenLayer.backPropagateCalculateErrorGradient();
            this.outputLayer.backPropagateOptimize();
            this.hiddenLayer.backPropagateOptimize();
        }
    }, {
        key: 'loadFromJson',
        value: function loadFromJson(json) {
            var weights = json.layers[1].weights;
            for (var i = 0; i < weights.length; i++) {
                //@TODO do this inside the layers
                this.hiddenLayer.weights[i] = weights[i];
            }
            weights = json.layers[2].weights;
            for (i = 0; i < weights.length; i++) {
                //@TODO do this inside the layers
                this.outputLayer.weights[i] = weights[i];
            }
        }
    }, {
        key: 'saveToJson',
        value: function saveToJson() {
            //@TODO use more future proof schema
            return {
                layers: [{}, //placeholder for future input layer info
                { weights: Array.from(this.hiddenLayer.weights) }, //@TODO do this inside the layers
                { weights: Array.from(this.outputLayer.weights) } //@TODO do this inside the layers
                ]
            }; //@TODO do this inside the layers
        }
    }]);

    return DeepNetwork;
}();

exports.default = DeepNetwork;